/*****************************************************************************\
 * TP CPE, 3ETI, TP synthese d'images
 * --------------
 *
 * Programme principal des appels OpenGL
\*****************************************************************************/



#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cmath>

#define GLEW_STATIC 1
#include <GL/glew.h>
#include <GL/glut.h>

#include "glutils.hpp"
#include "vec2.hpp"
#include "vec3.hpp"
#include "triangle_index.hpp"


/*****************************************************************************\
 * Variables globales
 *
 *
\*****************************************************************************/

//rotation de la camera
int rotation_x=0;
int rotation_y=0;

//identifiant du shader
GLuint shader_program_id;

GLuint vbo;


int counter_time=0;


/*****************************************************************************\
 * Fonctions GLUT
 *
 *
\*****************************************************************************/

//Fonction d'affichage
static void display_callback()
{
    //mise en place de la matrice de projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(30,1.0,0.1,20.0); //une matrice de perspective (affiche profondeur z entre [0.1,20])

    //mise en place de la matrice "monde"
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0.0f,0.0f,-4.0f); //deplacement de la camera vers l'arriere
    glRotatef(rotation_x,1.0f,0.0f,0.0f); //rotation suivant un axe
    glRotatef(rotation_y,0.0f,1.0f,0.0f); //rotation suivant l'autre axe


    //effacement des couleurs du fond d'ecran
    glClearColor(0.5, 0.6, 0.9, 1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


    glDrawArrays(GL_TRIANGLES, 0, 6); PRINT_OPENGL_ERROR();


    //Changement de buffer d'affichage pour eviter un effet de scintillement
    glutSwapBuffers();
    PRINT_OPENGL_ERROR(); //recherche d'erreurs opengl
}

/*****************************************************************************\
 * init                                                                      *
\*****************************************************************************/
static void init()
{

    vec3 p0(0,0,0);
    vec3 p1(1,0,0);
    vec3 p2(0,1,0);
    vec3 p3(0,0,1);

    float s=1.0f/sqrt(2.0f);
    vec3 n0(0,s,s);
    vec3 n1(0,0,1);
    vec3 n2(0,1,0);


    /*vec3 vertex[]={p0,p1,p2,p0,p1,p3,
                   n0,n0,n1,n0,n0,n2};*/

    glGenBuffers(1,&vbo); PRINT_OPENGL_ERROR();
    glBindBuffer(GL_ARRAY_BUFFER,vbo); PRINT_OPENGL_ERROR();
    glBufferData(GL_ARRAY_BUFFER,sizeof(vertex),vertex,GL_STATIC_DRAW); PRINT_OPENGL_ERROR();

    glEnableClientState(GL_VERTEX_ARRAY); PRINT_OPENGL_ERROR();
    glVertexPointer(3, GL_FLOAT, 0, 0); PRINT_OPENGL_ERROR();

    glEnableClientState(GL_NORMAL_ARRAY); PRINT_OPENGL_ERROR();
    long int offset_normal=18*sizeof(float);
    glNormalPointer(GL_FLOAT,0,buffer_offset(offset_normal)); PRINT_OPENGL_ERROR();

    // Chargement du shader
    shader_program_id = read_shader("shader.vert", "shader.frag");PRINT_OPENGL_ERROR();


    //activation de la gestion de la profondeur
    glEnable(GL_DEPTH_TEST); PRINT_OPENGL_ERROR();

}

/*****************************************************************************\
 * keyboard_callback                                                         *
\*****************************************************************************/
static void keyboard_callback(unsigned char key, int, int)
{
    //quitte le programme si on appuis sur les touches 'q', 'Q', ou 'echap'
    switch (key)
    {
    case 'q':
    case 'Q':
    case 27:
        exit(0);
    }
}

/*****************************************************************************\
 * special_callback                                                          *
\*****************************************************************************/
static void special_callback(int key, int,int)
{
  switch (key)
  {
  case GLUT_KEY_UP:
      rotation_x+=3; //rotation avec la touche du haut
      break;
  case GLUT_KEY_DOWN:
      rotation_x-=3; //rotation avec la touche du bas
      break;
  case GLUT_KEY_LEFT:
      rotation_y+=3; //rotation avec la touche de gauche
      break;
  case GLUT_KEY_RIGHT:
      rotation_y-=3; //rotation avec la touche de droite
      break;
  }

  //reactualisation de l'affichage
  glutPostRedisplay();
}

/*****************************************************************************\
 * timer_callback                                                            *
\*****************************************************************************/
static void timer_callback(int)
{
    //demande de rappel de cette fonction dans 25ms
    glutTimerFunc(25, timer_callback, 0);

    counter_time++;

    //reactualisation de l'affichage
    glutPostRedisplay();
}

int main(int argc, char** argv)
{
    //**********************************************//
    //Lancement des fonctions principales de GLUT
    //**********************************************//

    //initialisation
    glutInit(&argc, argv);

    //Mode d'affichage (couleur, gestion de profondeur, ...)
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

    //Taille de la fenetre a l'ouverture
    glutInitWindowSize(800, 800);

    //Titre de la fenetre
    glutCreateWindow("OpenGL");

    //Fonction de la boucle d'affichage
    glutDisplayFunc(display_callback);

    //Fonction de gestion du clavier
    glutKeyboardFunc(keyboard_callback);

    //Fonction des touches speciales du clavier (fleches directionnelles)
    glutSpecialFunc(special_callback);

    //Fonction d'appel d'affichage en chaine
    glutTimerFunc(25, timer_callback, 0);

    //Initialisation des fonctions OpenGL
    glewInit();

    //Notre fonction d'initialisation des donnees et chargement des shaders
    init();


    //Lancement de la boucle (infinie) d'affichage de la fenetre
    glutMainLoop();

    //Plus rien n'est execute apres cela

    return 0;
}

